// a dll to decode an 8 bit audio data stream into its fsk
// data symbols assuming 1200Hz = 0, 2200Hz = 1, 1200 bps
// and a sample rate of either 7200 or 9600

// definitions of calls
int filter (unsigned char thingy, int spb);

#include "nbfsk.h"

// uart variables
#define IDLE 1
#define START 2
#define RUN 4
#define RXFLAG 8
#define FRAMERR 16
#define STOP 32

static int lo1k2[21] = {
	0,	// -0.00000010
	0,	// -0.00000010
	189,	// 0.00578410
	347,	// 0.01061810
	0,	// 0.00000010
	-975,	// -0.02977210
	-1540,	// -0.04701110
	0,	// -0.00000010
	4119,	// 0.12572310
	8829,	// 0.26945810
	10922,	// 0.33333310
	8829,	// 0.26945810
	4119,	// 0.12572310
	0,	// -0.00000010
	-1540,	// -0.04701110
	-975,	// -0.02977210
	0,	// 0.00000010
	347,	// 0.01061810
	189,	// 0.00578410
	0,	// -0.00000010
	0};	// -0.00000010

static int hi1k2[21] = {
	0,	// 0.00000010
	84,	// 0.00256410
	0,	// 0.00000010
	-284,	// -0.00866910
	-691,	// -0.02110710
	-796,	// -0.02430910
	0,	// -0.00000010
	1992,	// 0.06080010
	4756,	// 0.14517310
	7209,	// 0.22001210
	8191,	// 0.25000010
	7209,	// 0.22001210
	4756,	// 0.14517310
	1992,	// 0.06080010
	0,	// -0.00000010
	-796,	// -0.02430910
	-691,	// -0.02110710
	-284,	// -0.00866910
	0,	// 0.00000010
	84,	// 0.00256410
	0};	// 0.00000010

int filter (unsigned char thingy, int spb)
{
int ac;
short input;
short output;
char result;

static short delay[5];
static short b[22];

static char rx = 0;
static short status = 1;
static int clock = 0;		// counter for sample
static int bit = 0;			// bit counter
int q;

	input = ((((short)thingy)-0x80)<<8);	// upscale the sample

	// we delay the input samples_per_bit/2 samples...
	for (q=0; q<(spb/2+1); q++)
		delay[q] = delay[q+1];

	// save new sample
		delay[spb/2] = input;

	// now we multiply todays sample with the delayed one
	ac = (int)delay[0]*(int)input*-1;

	// and finally, low pass filter the result
	// select filter depending on sample rate
	b[0] = ac>>15;
		ac = 0;
	if (spb < 7)
	{
		for (q=21; q>=0; q--)
		{
			ac += (int)b[q]*(int)lo1k2[q];
			b[q+1] = b[q];
		}
	}
	else
	{
		for (q=21; q>=0; q--)
		{
			ac += (int)b[q]*(int)hi1k2[q];
			b[q+1] = b[q];
		}
	}
	if (ac>0)
		result = 1;
	else
		result = 0;

	// now we can build a uart
	// the data is delivered in ten bit chunks...
	// in order,
	// start bit (0)
	// bit 0
	// ...
	// bit 7
	// stop bit (1)


	// see what our state is
	if (status & IDLE)		// we're idling
	{
		status &= ~RXFLAG;
		if (!result)				// falling edge of start bit
		{
			status &= ~IDLE;	// so we idle no longer
			status |= START;	// we're started
			bit = 0;					// reset bit counter
			clock = 0;				// and the clock count
		}
		// else we're still waiting for end of stop bit
	}
	else
	{
		if (status & START)		// aha, we got the falling edge
		{
			if ((clock <= spb/2) && (result))	// oops, false trigger...noise perhaps
			{
				status &= ~START;
				status |= IDLE;		// so drop back to idle mode
			}
			else
				clock++;					// otherwise, one more clock
			if (clock == spb/2)			// or are we now in mid start-bit
			{
				status &= ~START;
				status &= ~IDLE;
				status |= RUN;		// so now we're hot to trot
				clock = 0;				// reset counter
			}
		}
		else
		{
			if (status & RUN)		// we're reading data (allegedly)
			{
				if (clock < spb-1)		// time for a sample?
					clock++;				// not yet
				else
				{
					if (bit != 8)		// normal read
					{
						clock = 0;
						rx = rx>>1;
						if (result)
							rx |= 0x80;
						else
							rx &= 0x7f;
						bit ++;
					}
					else
					{
						if (! result)	// frame error
						{
							status |= FRAMERR;
						}
						else
						{
							status &= ~FRAMERR;
						}
						status |= IDLE;
						status |= RXFLAG;
						status &= ~RUN;
						status &= ~START;
					}
				}
			}
		}
	}

	output = (status<<8)+rx;
	return(output);
}

int _stdcall NbFskDecodeBytes (unsigned char * audio,
											long length,
											unsigned char * decode,
											int samples_per_bit)
{
	// this routine takes a block of 8 bit audio data and decodes it
	// using the bel202 fsk standard
	// only bytes which decode without a frame error are saved in 'decode'
	// the calling routine is responsible for ensuring that the decode
	// buffer is sufficiently large - there are about 60-80 samples per
	// byte of output
	// if 'clear' is set then the filter buffer is cleared before use


int q;
int bytes = 0;
struct rec1 {
	char rx;
	char status;
} rec;
union {
	short rex;
	rec1 rec;
} r;

	for (q=0; q<length; q++)
	{
		r.rex = filter(audio[q],samples_per_bit);
		if (r.rec.status & RXFLAG)        // aha - a byte is complete
		{
			if (!(r.rec.status & FRAMERR))  // and it has no frame error
			{
				decode[bytes++] = r.rec.rx;
			}
		}
	}
	return bytes;
}
